package client

import (
	"context"
	"log"
	"net/http"
	"os"
	"time"

	"github.com/wechatpay-apiv3/wechatpay-go/core"
	"github.com/wechatpay-apiv3/wechatpay-go/core/auth/verifiers"
	"github.com/wechatpay-apiv3/wechatpay-go/core/downloader"
	"github.com/wechatpay-apiv3/wechatpay-go/core/notify"
	"github.com/wechatpay-apiv3/wechatpay-go/core/option"
	"github.com/wechatpay-apiv3/wechatpay-go/utils"
)

var (
	// 支持微信支付 API v3 请求签名与应答验签能力的 HTTP Client
	client *core.Client
	// 提供回调通知验签与解密功能的回调通知处理器
	notifyHandler *notify.Handler
)

// Init 初始化 client 和 notifyHandler
func Init() {
	privateKeyPath := os.Getenv("WXPAY_MERCHANT_PRIVATE_KEY_PATH")
	mchPrivateKey, err := utils.LoadPrivateKeyWithPath(privateKeyPath)
	if err != nil {
		log.Fatalf("Fail to load private key from %s: %v", privateKeyPath, err)
	}

	if client, err = core.NewClient(
		context.Background(),
		option.WithWechatPayAutoAuthCipher(
			os.Getenv("WXPAY_MERCHANT_ID"),
			os.Getenv("WXPAY_MERCHANT_CERTIFICATE_SERIAL_NUMBER"),
			mchPrivateKey,
			os.Getenv("WXPAY_MERCHANT_APIV3_KEY"),
		),
		option.WithHTTPClient(&http.Client{Timeout: time.Second * 3}),
	); err != nil {
		log.Fatalf("Fail to new client: %v", err)
	}
	certVisitor := downloader.MgrInstance().GetCertificateVisitor(os.Getenv("WXPAY_MERCHANT_ID"))
	notifyHandler = notify.NewNotifyHandler(
		os.Getenv("WXPAY_MERCHANT_APIV3_KEY"),
		verifiers.NewSHA256WithRSAVerifier(certVisitor),
	)
}

// GetClient 获取 client
func GetClient() *core.Client {
	if client == nil {
		log.Fatalf("Client has not been initialized")
	}
	return client
}

// GetNotifyHandler 获取 notifyHandler
func GetNotifyHandler() *notify.Handler {
	if notifyHandler == nil {
		log.Fatal("Notify handler has not been initialized")
	}
	return notifyHandler
}
