package handler

import (
	"fmt"
	"io/ioutil"
	"log"
	"net/http"

	"github.com/pkg/errors"
	"tencent.com/wxp/credit-parking-demo/client"
	"tencent.com/wxp/credit-parking-demo/model"
	"tencent.com/wxp/credit-parking-demo/util"
)

// HandleOrderRequest 处理订单/交易相关请求
func HandleOrderRequest(w http.ResponseWriter, r *http.Request) {
	if r.Method == http.MethodGet {
		// 处理主动查单请求
		handleQueryOrder(w, r)
	} else if r.Method == http.MethodPost {
		// 处理订单扣款状态变更请求
		handleOrderStatusChange(w, r)
	} else {
		// 其他情况返回 405 Method Not Allowed 错误
		http.Error(w, http.StatusText(http.StatusMethodNotAllowed), http.StatusMethodNotAllowed)
	}
}

func handleQueryOrder(w http.ResponseWriter, r *http.Request) {
	outTradeNo := r.URL.Query().Get("out_trade_no")
	if outTradeNo == "" {
		err := errors.Errorf("<out_trade_no> is required for querying order")
		log.Print(err.Error())
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}

	apiResult, err := client.GetClient().Get(
		r.Context(),
		fmt.Sprintf("https://api.mch.weixin.qq.com/v3/vehicle/transactions/out-trade-no/%s", outTradeNo),
	)

	if err != nil {
		err = errors.Errorf("Fail to query order: %v", err)
		log.Print(err.Error())
		http.Error(w, err.Error(), http.StatusInternalServerError)
		return
	}

	respBody, err := ioutil.ReadAll(apiResult.Response.Body)
	if err != nil {
		err = errors.Errorf("Fail to read from response body: %v", err)
		log.Print(err.Error())
		http.Error(w, err.Error(), http.StatusInternalServerError)
		return
	}

	log.Printf("Resp: %s", respBody)
}

func handleOrderStatusChange(w http.ResponseWriter, r *http.Request) {
	log.Println("=============== Order status callback ===================")
	req := model.OrderStatusNotifyRequest{}
	if _, err := client.GetNotifyHandler().ParseNotifyRequest(r.Context(), r, &req); err != nil {
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}
	log.Printf("Req: %v", util.ToString(req))

	switch req.TradeState {
	case string("SUCCESS"):
		// 订单支付成功
	case string("ACCEPTED"):
		// 订单已受理，后续一段时间内需要主动查询订单
		break
	case string("PAY_FAIL"):
		// 订单支付失败，需要记录保存订单信息，并进行商户方支付失败场景的处理
		break
	case string("REFUND"):
		// 订单转入退款，需要记录保存订单信息，并进行商户方退款场景的处理
		break
	default:
		// 未知订单状态，需要记录保存订单信息
		break
	}

	util.WriteJSON(w, struct {
		Code string `json:"code"`
	}{
		Code: "SUCCESS",
	})
}
