package handler

import (
	"encoding/json"
	"fmt"
	"io/ioutil"
	"log"
	"net/http"
	"os"
	"time"

	"github.com/pkg/errors"
	"tencent.com/wxp/credit-parking-demo/client"
	"tencent.com/wxp/credit-parking-demo/dao"
	"tencent.com/wxp/credit-parking-demo/error"
	"tencent.com/wxp/credit-parking-demo/model"
	"tencent.com/wxp/credit-parking-demo/util"
)

// HandleParkingRequest 处理停车相关请求
func HandleParkingRequest(w http.ResponseWriter, r *http.Request) {
	if r.Method != http.MethodPost && r.Method != http.MethodPut {
		// 返回 405 Method Not Allowed 错误
		http.Error(w, http.StatusText(http.StatusMethodNotAllowed), http.StatusMethodNotAllowed)
		return
	}

	body, err := ioutil.ReadAll(r.Body)
	if err != nil {
		err = errors.Errorf("Fail to read request body: body=%v, err=%v", r.Body, err)
		log.Print(err.Error())
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}

	params := model.ParkingParams{}
	if err = json.Unmarshal(body, &params); err != nil {
		err = errors.Errorf("Fail to unmarshal body: %v", err)
		log.Print(err.Error())
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}

	if params.PlateNumber == "" {
		err = errors.Errorf("Request field <plate_number> is required")
		log.Print(err.Error())
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}

	if r.Method == http.MethodPost {
		// 处理停车入场请求
		handleCreateParking(w, r, params)
	} else {
		// 处理停车变更请求，此处即代表车主离场
		handleStopParking(w, r, params)
	}
}

func handleCreateParking(w http.ResponseWriter, r *http.Request, params model.ParkingParams) {
	// 判断是否已经停车，若已停车，直接返回错误
	if dao.GetParkingStatus(params.PlateNumber) {
		err := errors.Errorf("Vehicle %s is already parking now", params.PlateNumber)
		log.Print(err.Error())
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}

	// 生成商户方的停车 ID
	// 保存用户停车入场信息、生成商户侧入场标识 ID 的具体实现需要商户方根据现实情况进行处理
	outParkingNo := util.GetRandIntStr()
	// 构造并发起创建停车请求
	req := model.CreateParkingRequest{
		PlateNumber:  params.PlateNumber,
		PlateColor:   params.PlateColor,
		ParkingName:  params.ParkingName,
		StartTime:    params.StartTime,
		FreeDuration: params.FreeDuration,
		NotifyURL:    fmt.Sprintf("%s/service", os.Getenv("DOMAIN")),
		OutParkingNo: outParkingNo,
	}
	reqBody, err := json.Marshal(&req)
	if err != nil {
		err = errors.Errorf("Fail to marshal request: %v", req)
		log.Print(err.Error())
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}
	log.Printf("Req: %s", string(reqBody))

	apiResult, err := client.GetClient().Post(
		r.Context(),
		"https://api.mch.weixin.qq.com/v3/vehicle/parking/parkings",
		reqBody,
	)
	if err != nil {
		err = errors.Errorf("Fail to create parking service: %v", err)
		log.Print(err.Error())
		http.Error(w, err.Error(), http.StatusInternalServerError)
		return
	}

	respBody, err := ioutil.ReadAll(apiResult.Response.Body)
	if err != nil {
		err = errors.Errorf("Fail to read from response body: %v", err)
		log.Print(err.Error())
		http.Error(w, err.Error(), http.StatusInternalServerError)
		return
	}
	log.Printf("Resp: %s", string(respBody))

	resp := model.CreateParkingResponse{}
	if err = json.Unmarshal(respBody, &resp); err != nil {
		err = errors.Errorf("Fail to unmarshal from response body: %v", err)
		log.Print(err.Error())
		http.Error(w, err.Error(), http.StatusInternalServerError)
		return
	}

	// 判断用户服务状态
	if resp.State == "NORMAL" { // nolint:goconst
		// 更新用户停车信息，入场成功
		dao.SaveServiceStatus(params.PlateNumber, true)
		dao.SaveParkingID(params.PlateNumber, resp.ParkingID)
		dao.SaveOutParkingNo(params.PlateNumber, outParkingNo)
	} else {
		// 更新用户停车信息，入场失败
		dao.SaveServiceStatus(params.PlateNumber, false)
		err = errors.Errorf("Vehicle is blocked for reason: %v", resp.BlockReason)
		log.Print(err.Error())
		util.WriteError(w, http.StatusForbidden, error.Error{
			Code:    error.PermissionDenied,
			Message: err.Error(),
			Details: struct {
				Reason string `json:"reason"`
			}{
				Reason: resp.BlockReason,
			},
		})
	}
}

func handleStopParking(w http.ResponseWriter, r *http.Request, params model.ParkingParams) {
	parkingID := dao.GetParkingID(params.PlateNumber)
	// 检查用户停车入场状态，或未停车，则返回错误
	if parkingID == "" {
		err := errors.Errorf("Vehicle %s is not parking now", params.PlateNumber)
		log.Print(err.Error())
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}

	// 检查用户服务状态，若未开通服务，则返回错误
	if !dao.GetServiceStatus(params.PlateNumber) {
		err := errors.Errorf("Vehicle %s is not in credit parking service", params.PlateNumber)
		log.Print(err.Error())
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}

	// 构造并发起申请扣款请求
	req := model.CreateTransactionRequest{
		AppID:         os.Getenv("WXPAY_MERCHANT_APP_ID"),
		Description:   "停车场扣费",
		Attach:        "深圳分店",
		GoodsTag:      "WXG",
		ProfitSharing: "N",
		OutTradeNo:    dao.GetOutParkingNo(params.PlateNumber),
		TradeScene:    "PARKING",
		NotifyURL:     fmt.Sprintf("%s/orders", os.Getenv("DOMAIN")),
		Amount: model.Amount{
			Total:    1,
			Currency: "CNY",
		},
		ParkingInfo: model.ParkingInfo{
			ParkingID:        parkingID,
			PlateNumber:      params.PlateNumber,
			PlateColor:       params.PlateColor,
			StartTime:        params.StartTime,
			EndTime:          params.StartTime.Add(time.Hour),
			ChargingDuration: 3600,
			ParkingName:      params.ParkingName,
			DeviceID:         util.GetRandIntStr(),
		},
	}

	reqBody, _ := json.Marshal(&req)
	log.Printf("Req: %s", string(reqBody))
	apiResult, err := client.GetClient().Post(
		r.Context(),
		"https://api.mch.weixin.qq.com/v3/vehicle/transactions/parking",
		reqBody,
	)
	if err != nil {
		err = errors.Errorf("Fail to create transaction: %v", err)
		log.Print(err.Error())
		http.Error(w, err.Error(), http.StatusInternalServerError)
		return
	}

	respBody, err := ioutil.ReadAll(apiResult.Response.Body)
	if err != nil {
		err = errors.Errorf("Fail to read from response body: %v", err)
		log.Print(err.Error())
		http.Error(w, err.Error(), http.StatusInternalServerError)
		return
	}
	log.Printf("Resp: %s", respBody)

	resp := model.CreateTransactionResponse{}
	if err = json.Unmarshal(respBody, &resp); err != nil {
		err = errors.Errorf("Fail to unmarshal from response body: %v", err)
		log.Print(err.Error())
		http.Error(w, err.Error(), http.StatusInternalServerError)
		return
	}

	// 记录用户扣款信息

	// 示例代码中通过删除车辆停车 ID 的方式实现离场操作，实际场景中商户需要根据自身情况进行用户停车状态的更新
	dao.RemoveParkingID(params.PlateNumber)
}
