package handler

import (
	"fmt"
	"io/ioutil"
	"log"
	"net/http"
	"os"

	"github.com/pkg/errors"
	"tencent.com/wxp/credit-parking-demo/client"
	"tencent.com/wxp/credit-parking-demo/dao"
	"tencent.com/wxp/credit-parking-demo/model"
	"tencent.com/wxp/credit-parking-demo/util"
)

// HandleServiceRequest 处理车主服务相关请求
func HandleServiceRequest(w http.ResponseWriter, r *http.Request) {
	if r.Method == http.MethodGet {
		// 处理主动查询用户车辆服务状态请求
		handleQueryServiceStatus(w, r)
	} else if r.Method == http.MethodPost {
		// 处理停车入场状态变更通知
		handleServiceStatusChange(w, r)
	} else {
		// 其他情况返回 405 Method Not Allowed 错误
		http.Error(w, http.StatusText(http.StatusMethodNotAllowed), http.StatusMethodNotAllowed)
	}
}

func handleQueryServiceStatus(w http.ResponseWriter, r *http.Request) {
	appID := os.Getenv("WXPAY_MERCHANT_APP_ID")
	query := r.URL.Query()
	plateNumber := query.Get("plate_number")
	plateColor := query.Get("plate_color")
	openID := query.Get("openid")
	if plateNumber == "" || plateColor == "" || openID == "" {
		err := errors.Errorf("<plate_number>, <plate_color>, <openid> are required for querying service status")
		log.Print(err.Error())
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}

	apiResult, err := client.GetClient().Get(
		r.Context(),
		fmt.Sprintf(
			"https://api.mch.weixin.qq.com/v3/vehicle/parking/services/find?appid=%s&plate_number=%s&plate_color=%s&openid=%s",
			appID,
			plateNumber,
			plateColor,
			openID,
		),
	)
	if err != nil {
		err = errors.Errorf("Fail to query service status: %v", err)
		log.Print(err.Error())
		http.Error(w, err.Error(), http.StatusInternalServerError)
		return
	}

	respBody, err := ioutil.ReadAll(apiResult.Response.Body)
	if err != nil {
		err = errors.Errorf("Fail to read from response body: %v", err)
		log.Print(err.Error())
		http.Error(w, err.Error(), http.StatusInternalServerError)
		return
	}

	log.Printf("Resp: %s", respBody)
}

func handleServiceStatusChange(w http.ResponseWriter, r *http.Request) {
	log.Println("=============== Service status callback ===================")
	req := model.ServiceStatusNotifyRequest{}
	if _, err := client.GetNotifyHandler().ParseNotifyRequest(r.Context(), r, &req); err != nil {
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}
	log.Printf("Req: %v", util.ToString(req))

	// 更新用户服务状态
	if req.ParkingState == "NORMAL" { // nolint:goconst
		dao.SaveServiceStatus(req.PlateNumber, true)
	} else {
		dao.SaveServiceStatus(req.PlateNumber, false)
	}
}
